// -------------------------------------------------------------------------
//   qSOAPman
//   Copyright (C) 2004 by Jan Schreiber
//   mail@jschreiber.com
//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program; if not, write to the
//   Free Software Foundation, Inc.,
//   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
// -------------------------------------------------------------------------

#include "config.h"
#include "mainsoapwidget.h"
#include "soapsyntaxhighlighter.h"
#include "simpleparameterdialog.h"

#include <qlineedit.h>
#include <qtextedit.h>
#include <qtabwidget.h>
#include <qcursor.h>
#include <qapplication.h>
#include <qstring.h>
#include <qspinbox.h>
#include <qcombobox.h>
#include <qcheckbox.h>
#include <qmessagebox.h>
#include <qurl.h>
#include <qxml.h>

MainSOAPWidget::MainSOAPWidget( QWidget* parent, const char* name, WFlags fl )
	: SOAPWidget( parent, name, fl )
{
        connect( &http, SIGNAL( done( bool ) ), this, SLOT( SOAPdoneSlot( bool ) ) );
	connect( &http, SIGNAL( responseHeaderReceived( const QHttpResponseHeader& ) ),
		this, SLOT( setResponseHeader( const QHttpResponseHeader& ) ) );
}

MainSOAPWidget::~MainSOAPWidget()
{
 }


QString MainSOAPWidget::getServer() const { return serviceEdit->text(); };
QString MainSOAPWidget::getSOAPAction() const { return SOAPActionEdit->text(); }
QString MainSOAPWidget::getHeader() const { return headerEdit->text(); }
QString MainSOAPWidget::getMessage() const { return messageEdit->text(); }
QString MainSOAPWidget::getProxy() const { return proxyEdit->text(); }
int MainSOAPWidget::getProxyPort() const { return proxyPortSpinBox->value(); }
QString MainSOAPWidget::getAdditionalNamespaces() const { return namespacesEdit->text(); }
bool MainSOAPWidget::getSendRaw() const { return sendRawCheckBox->isChecked(); }
bool MainSOAPWidget::getShowHttpHeaders() const { return showHeadersCheckBox->isChecked(); }
QString MainSOAPWidget::getHttpContentType() const { return contentTypeComboBox->currentText(); }

void MainSOAPWidget::setServer( const QString &s ) { serviceEdit->setText( s ); }
void MainSOAPWidget::setSOAPAction( const QString &s ) { SOAPActionEdit->setText( s ); }
void MainSOAPWidget::setHeader( const QString &s ) { headerEdit->setText( s ); }
void MainSOAPWidget::setMessage( const QString &s ) { messageEdit->setText( s ); }
void MainSOAPWidget::setProxy ( const QString &s ) { proxyEdit->setText( s ); }
void MainSOAPWidget::setProxyPort( int i ) { proxyPortSpinBox->setValue( i ); }
void MainSOAPWidget::setAdditionalNamespaces( const QString &s ) { namespacesEdit->setText( s ); }
void MainSOAPWidget::setSendRaw( bool b ) { sendRawCheckBox->setChecked( b ); }
void MainSOAPWidget::setShowHttpHeaders( bool b ) { showHeadersCheckBox->setChecked( b ); }
void MainSOAPWidget::setHttpContentType( const QString &s ) { contentTypeComboBox->setCurrentText( s ); }

void MainSOAPWidget::clearAllSlot()
{
	serviceEdit->clear();
	SOAPActionEdit->clear();
	headerEdit->clear();
	messageEdit->clear();
	resultEdit->clear();
	previewEdit->clear();
}

void MainSOAPWidget::addParameterSlot()
{
	SimpleParameterDialog *dlg = new SimpleParameterDialog( this, APP_NAME );

        if ( dlg->exec() == QDialog::Accepted ) {
		QString parameter = "<";
		parameter.append( dlg->nameEdit->text() );
		if ( !dlg->typeBox->currentText().isEmpty() )
		{
			parameter.append( " xsi:type=\"" );
			parameter.append( dlg->typeBox->currentText() );
			parameter.append( "\"" );
		}
		if ( !dlg->valueEdit->text().isEmpty() )
		{
			parameter.append( ">" );
			parameter.append( dlg->valueEdit->text() );
			parameter.append( "</ ");
			parameter.append( dlg->nameEdit->text() );
			parameter.append( ">" );
		} else
			parameter.append( " />" );
		messageEdit->insert( parameter );
        }
        delete dlg;

}

QString MainSOAPWidget::createSOAPMessage()
{
	if ( sendRawCheckBox->isChecked() ) return messageEdit->text();
	QString ret;

	// FIXME: hard-coded XML
	ret = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n<SOAP-ENV:Envelope  xmlns:SOAP-ENV=\"http://schemas.xmlsoap.org/soap/envelope/\" ";

	if ( !namespacesEdit->text().isEmpty() )
	{
		ret += namespacesEdit->text();
	}
	ret += ">";
	if ( !headerEdit->text().isEmpty() )
	{
		ret += "  <SOAP-ENV:Header xmlns:env=\"http://www.w3.org/2003/05/soap-envelope\">"
			+ headerEdit->text() +
			"  </SOAP-ENV:Header>";
	}
	ret += "  <SOAP-ENV:Body>"
		+ messageEdit->text() +
		"  </SOAP-ENV:Body>"
		"</SOAP-ENV:Envelope>";
	return ret;
}

void MainSOAPWidget::setResponseHeader( const QHttpResponseHeader & resp )
{
	response = resp;
}

void MainSOAPWidget::sendMessageSlot()
{
	QApplication::setOverrideCursor( QCursor( Qt::WaitCursor ) );

	// Split the URL into hostname and filename/path
	QUrl url( serviceEdit->text() );

	QHttpRequestHeader header;

	// if we post through a proxy server, the post request must contain
	// the full url
	if ( !proxyEdit->text().isEmpty() )
		header.setRequest( "POST", url.toString() );
	else
        	header.setRequest( "POST", url.path() );

	if ( url.hasPort() )
		header.setValue( "Host", url.host().append( ":" ).append( url.port() ) );
	else
        	header.setValue( "Host", url.host() );

	if ( !SOAPActionEdit->text().isEmpty())
	{
		header.setValue( "SOAPAction", SOAPActionEdit->text() );
	}

	if ( !contentTypeComboBox->currentText().isEmpty() )
        	header.setContentType( contentTypeComboBox->currentText() );

	if ( !proxyEdit->text().isEmpty() )
		http.setHost( proxyEdit->text(), proxyPortSpinBox->value() );
	else
		if ( url.hasPort() )
			http.setHost( url.host(), url.port() );
		else
        		http.setHost( url.host() );


	QCString mesg = createSOAPMessage().utf8();
	QByteArray ba;
	ba.duplicate( mesg.data(), mesg.length() );
	emit startRequest();
	http.request( header, ba );

	// set focus to result edit
	resultEdit->clear();
	tabWidget->setCurrentPage( 2 );
}

void MainSOAPWidget::SOAPdoneSlot( bool error )
{
	emit endRequest();
	QApplication::restoreOverrideCursor();
	QString resultStr = http.readAll();
	if ( error ) {
		QMessageBox::warning( this,
			tr("HTTP Error"),
			QString( tr("Error sending the SOAP message:\n") ).append( http.errorString() ) );
			resultEdit->setText( resultStr );
			return;
	}
	QString header_str;
	if ( showHeadersCheckBox->isChecked() )
	{
		header_str.append( QString("<b>HTTP/%1.%2 %3 %4</b><br>").arg( response.majorVersion() ).arg( response.minorVersion() ).arg( response.statusCode() ).arg( response.reasonPhrase() ) );
		QStringList headers = response.keys();
		for ( QStringList::Iterator it = headers.begin(); it != headers.end(); ++it ) 
		{
        		header_str.append( "<b>" + (*it) + ":</b> " + response.value( *it ) + "<br>" );
    		}
		if ( !header_str.isEmpty() ) header_str.append( "<br>" );
	}

	QString prettyString;
        if ( prettyPrint( resultStr, prettyString ) )
	{
		resultEdit->setText( header_str + prettyString );
	} else {
		resultEdit->setText( header_str + resultStr );
	}
}

void MainSOAPWidget::previewSlot()
{
	QString prettyString, result = createSOAPMessage();
	
	if ( prettyPrint( result, prettyString ) )
	{
		previewEdit->setText( prettyString );
		tabWidget->setCurrentPage( 1 );
	} else 
	{
		QMessageBox::warning( this,
		tr("XML Error"),
		QString( tr("The XML parser returned an error.\nPlease check your input!") ) );
	}
}

void MainSOAPWidget::editSlot()
{
	tabWidget->setCurrentPage( 0 );
	messageEdit->setFocus();
}

bool MainSOAPWidget::prettyPrint( const QString& xml, QString& result )
{
	SOAPSyntaxHighlighter handler;

        QXmlInputSource source;
	source.setData( xml );
        QXmlSimpleReader reader;
	reader.setFeature( "http://xml.org/sax/features/namespaces", TRUE );
	reader.setFeature( "http://xml.org/sax/features/namespace-prefixes", TRUE );
	reader.setFeature( "http://trolltech.com/xml/features/report-start-end-entity", FALSE );

        reader.setContentHandler( &handler );
	reader.setLexicalHandler ( &handler );

	bool ret = reader.parse( source );
	result = handler.getResult();
	return ret;
}
