// -------------------------------------------------------------------------
//   qSOAPman
//   Copyright (C) 2004 by Jan Schreiber
//   mail@jschreiber.com
//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program; if not, write to the
//   Free Software Foundation, Inc.,
//   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
// -------------------------------------------------------------------------

#ifndef MAINSOAPWIDGET_H
#define MAINSOAPWIDGET_H

#include "soapwidget.h"

#include <qstring.h>
#include <qhttp.h>

/**
* This class defines the functionality of the "main window".
* It's a tabbed widget with tabs for message creation, message preview and
* message results.
*/
class MainSOAPWidget : public SOAPWidget
{
	Q_OBJECT
public:
	MainSOAPWidget( QWidget* parent = 0, const char* name = 0, WFlags fl = 0 );
	~MainSOAPWidget();

	// Access functions
	QString getServer() const;
	QString getSOAPAction() const;
	QString getHeader() const;
	QString getMessage() const;
	QString getProxy() const;
	int getProxyPort() const;
	QString getAdditionalNamespaces() const;
	bool getSendRaw() const;
	bool getShowHttpHeaders() const;
	QString getHttpContentType() const;

	void setServer( const QString & );
	void setSOAPAction( const QString & );
	void setHeader( const QString & );
	void setMessage( const QString & );
	void setProxy ( const QString & );
	void setProxyPort( int );
	void setAdditionalNamespaces( const QString & );
	void setSendRaw( bool );
	void setShowHttpHeaders( bool );
	void setHttpContentType( const QString & );

public slots:
	/**
	* Opens the parameter query dialog and inserts the new parameter at the current
	* cursor position in the messageEdit, if the user accepts the input.
	*/
	void addParameterSlot();

	/**
	* Clears the contents of all text fields (also on the result and preview tabs)
	*/
	void clearAllSlot();

	void editSlot();

	/**
	* Generates the SOAP message and sets the focus to the "preview" tab.
	*/
	void previewSlot();
	
	bool prettyPrint( const QString& xml, QString& result );

signals:
	void startRequest();
	void endRequest();
	
private slots:
	/**
	* This slot checks the message/header/service edit contents and sends a message to
	* the specified service, if all fields are valid (FIXME: right now, this only means "not empty")
	*/
	void sendMessageSlot();
	void SOAPdoneSlot( bool error );
	void setResponseHeader( const QHttpResponseHeader & resp );

private:
	QString createSOAPMessage();
	QHttp http;
	QHttpResponseHeader response;
};

#endif
