// -------------------------------------------------------------------------
//   qSOAPman
//   Copyright (C) 2004 by Jan Schreiber
//   mail@jschreiber.com
//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program; if not, write to the
//   Free Software Foundation, Inc.,
//   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
// -------------------------------------------------------------------------


#include "config.h"
#include "qsoapman.h"

#include <qimage.h>
#include <qpixmap.h>
#include <qtoolbar.h>
#include <qtoolbutton.h>
#include <qpopupmenu.h>
#include <qmenubar.h>
#include <qtextedit.h>
#include <qfile.h>
#include <qfiledialog.h>
#include <qstatusbar.h>
#include <qmessagebox.h>
//#include <qprinter.h>
#include <qapplication.h>
#include <qaccel.h>
#include <qtextstream.h>
#include <qpainter.h>
#include <qpaintdevicemetrics.h>
#include <qwhatsthis.h>
#include <qregexp.h>
#include <qdom.h>

#include "filesave.xpm"
#include "fileopen.xpm"
//#include "fileprint.xpm"

qsoapman::qsoapman()
    : QMainWindow( 0, "qsoapman", WDestructiveClose )
{
    //printer = new QPrinter;
    QPixmap openIcon, saveIcon; //, printIcon;

    QToolBar * fileTools = new QToolBar( this, "session operations" );
    fileTools->setLabel( tr("Session Operations") );

    openIcon = QPixmap( fileopen );
    QToolButton * fileOpen
	= new QToolButton( openIcon, tr("Open Session"), QString::null,
			   this, SLOT( choose() ), fileTools, "open file" );

    saveIcon = QPixmap( filesave );
    QToolButton * fileSave
	= new QToolButton( saveIcon, tr("Save Session"), QString::null,
			   this, SLOT( save() ), fileTools, "save file" );

    /*printIcon = QPixmap( fileprint );
    QToolButton * filePrint
	= new QToolButton( printIcon, tr("Print File"), QString::null,
			   this, SLOT(print()), fileTools, "print file" );
	*/

    (void)QWhatsThis::whatsThisButton( fileTools );

    QString fileOpenText = tr("<p><img source=\"fileopen\"> "
	         "Click this button to open a <em>new file</em>. <br>"
                 "You can also select the <b>Open</b> command "
                 "from the <b>File</b> menu.</p>");

    QWhatsThis::add( fileOpen, fileOpenText );

    QMimeSourceFactory::defaultFactory()->setPixmap( "fileopen", openIcon );

    QString fileSaveText = tr("<p>Click this button to save the current "
                 "session. You will be prompted for a file name.\n"
                 "You can also select the <b>Save</b> command "
                 "from the <b>File</b> menu.</p>");

    QWhatsThis::add( fileSave, fileSaveText );

    /*
    QString filePrintText = tr("Click this button to print the file you "
                 "are editing.\n You can also select the Print "
                 "command from the File menu.");

    QWhatsThis::add( filePrint, filePrintText );
	*/

    QPopupMenu * file = new QPopupMenu( this );
    menuBar()->insertItem( tr("&File"), file );


    file->insertItem( tr("&New Session"), this, SLOT(newDoc()), CTRL+Key_N );

    int id;
    id = file->insertItem( openIcon, tr("&Open Session..."),
			   this, SLOT(choose()), CTRL+Key_O );
    file->setWhatsThis( id, fileOpenText );

    id = file->insertItem( saveIcon, tr("&Save Session"),
			   this, SLOT(save()), CTRL+Key_S );
    file->setWhatsThis( id, fileSaveText );

    id = file->insertItem( tr("Save Session &As..."), this, SLOT(saveAs()) );
    file->setWhatsThis( id, fileSaveText );

    /*file->insertSeparator();

    id = file->insertItem( printIcon, tr("&Print..."),
			   this, SLOT(print()), CTRL+Key_P );
    file->setWhatsThis( id, filePrintText );*/

    file->insertSeparator();

    file->insertItem( tr("&Close Session"), this, SLOT(close()), CTRL+Key_W );

    file->insertItem( tr("&Quit"), qApp, SLOT( closeAllWindows() ), CTRL+Key_Q );

    menuBar()->insertSeparator();

    QPopupMenu * help = new QPopupMenu( this );
    menuBar()->insertItem( tr("&Help"), help );

    help->insertItem( tr("&About"), this, SLOT(about()), Key_F1 );
    help->insertItem( tr("About &Qt"), this, SLOT(aboutQt()) );
    help->insertSeparator();
    help->insertItem( tr("What's &This"), this, SLOT(whatsThis()), SHIFT+Key_F1 );

    soapwidget = new MainSOAPWidget( this, "soapwidget" );
    setCentralWidget( soapwidget );
    statusBar()->message( tr("Ready"), 2000 );
    
    connect( soapwidget, SIGNAL( startRequest() ), this, SLOT( startRequestSlot() ) );
    connect( soapwidget, SIGNAL( endRequest() ), this, SLOT( endRequestSlot() ) );
}


qsoapman::~qsoapman()
{
	//delete printer;
}

void qsoapman::startRequestSlot()
{
	timer.start();
}

void qsoapman::endRequestSlot()
{
	int queryTime = timer.elapsed();
	statusBar()->message( tr("The last query took %1 msecs").arg( queryTime ) );
}

void qsoapman::newDoc()
{
    qsoapman *ed = new qsoapman;
    ed->setCaption( APP_NAME );
    ed->show();
}

void qsoapman::choose()
{
    QString fn = QFileDialog::getOpenFileName( QString::null, QString::null,
					       this);
    if ( !fn.isEmpty() )
	load( fn );
    else
	statusBar()->message( tr("Loading aborted"), 2000 );
}


void qsoapman::load( const QString &fileName )
{
	QFile file( fileName );

	QDomDocument doc( "configuration" );
	if ( !file.open( IO_ReadOnly ) )
		return;
	if ( !doc.setContent( &file ) ) {
		file.close();
		return;
	}
	file.close();

	// get the header information from the DOM
	QDomElement root = doc.documentElement();
	QDomNode node;
	QDomText textChild;
	node = root.firstChild();
	node = node.firstChild();
	while ( !node.isNull() )
	{
		if ( node.isElement() && node.nodeName() == "server" )
		{
			textChild = node.firstChild().toText();
                	if ( !textChild.isNull() )
				soapwidget->setServer( decodeSGMLEntities( textChild.nodeValue() ) );
        	} else
		if ( node.isElement() && node.nodeName() == "soapaction" )
		{
			textChild = node.firstChild().toText();
                	if ( !textChild.isNull() )
				soapwidget->setSOAPAction( decodeSGMLEntities( textChild.nodeValue() ) );
		}  else
		if ( node.isElement() && node.nodeName() == "header" )
		{
			textChild = node.firstChild().toText();
                	if ( !textChild.isNull() )
				soapwidget->setHeader( decodeSGMLEntities( textChild.nodeValue() ) );
		} else
		if ( node.isElement() && node.nodeName() == "message" )
		{
			textChild = node.firstChild().toText();
                	if ( !textChild.isNull() )
				soapwidget->setMessage( decodeSGMLEntities( textChild.nodeValue() ) );
		} else
		if ( node.isElement() && node.nodeName() == "proxy" )
		{
			textChild = node.firstChild().toText();
                	if ( !textChild.isNull() )
				soapwidget->setProxy( decodeSGMLEntities( textChild.nodeValue() ) );
		} else
		if ( node.isElement() && node.nodeName() == "proxy-port" )
		{
			textChild = node.firstChild().toText();
                	if ( !textChild.isNull() )
				soapwidget->setProxyPort( textChild.nodeValue().toInt() );
		} else
		if ( node.isElement() && node.nodeName() == "additional-namespaces" )
		{
			textChild = node.firstChild().toText();
                	if ( !textChild.isNull() )
				soapwidget->setAdditionalNamespaces( decodeSGMLEntities( textChild.nodeValue() ) );
		} else
		if ( node.isElement() && node.nodeName() == "send-raw" )
		{
			textChild = node.firstChild().toText();
                	if ( !textChild.isNull() )
				soapwidget->setSendRaw( textChild.nodeValue() == "1" );
		} else
		if ( node.isElement() && node.nodeName() == "show-headers" )
		{
			textChild = node.firstChild().toText();
                	if ( !textChild.isNull() )
				soapwidget->setShowHttpHeaders( textChild.nodeValue() == "1" );
		} else
		if ( node.isElement() && node.nodeName() == "http-content-type" )
		{
			textChild = node.firstChild().toText();
                	if ( !textChild.isNull() )
				soapwidget->setHttpContentType( decodeSGMLEntities(  textChild.nodeValue() ) );
		}
		node = node.nextSibling();
	}

	setCaption( fileName );
	statusBar()->message( tr("Loaded session %1").arg(fileName), 2000 );
}


QString qsoapman::encodeSGMLEntities( const QString &s ) const
{
	QString data = s;
	data.replace( "&", "&amp;" ); // This _must_ come first
	data.replace( "<", "&lt;" );
	data.replace( ">", "&gt;" );
	data.replace( "\"", "&quot;" );
	data.replace( "'", "&apos;" );
	return data;
}

QString qsoapman::decodeSGMLEntities( const QString &s ) const
{
	QString data = s;
	data.replace( "&amp;", "&" );
	data.replace( "&lt;", "<" );
	data.replace( "&gt;", ">" );
	data.replace( "&quot;", "\"" );
	data.replace( "&apos;", "'" );
	return data;
}


void qsoapman::save()
{
	if ( filename.isEmpty() ) {
		saveAs();
		return;
	}

	QFile f( filename );
	if ( !f.open( IO_WriteOnly ) ) {
	statusBar()->message( tr("Could not write session to %1").arg(filename),
			      2000 );
		return;
	}

	// FIXME: write the file using DOM
	QTextStream t( &f );
	t << "<?xml version=\"1.0\"?> \n";
	t << "<qsoapman-config version=\"0.1\">\n";
	t << "<soapsession>\n";
	t << "<server>" << encodeSGMLEntities( soapwidget->getServer() ) << "</server>\n";
	t << "<soapaction>" << encodeSGMLEntities( soapwidget->getSOAPAction() ) << "</soapaction>\n";
	t << "<header>" << encodeSGMLEntities( soapwidget->getHeader() ) << "</header>\n";
	t << "<message>" << encodeSGMLEntities( soapwidget->getMessage() ) << "</message>\n";
	t << "<proxy>" << encodeSGMLEntities( soapwidget->getProxy() ) << "</proxy>\n";
	t << "<proxy-port>" << soapwidget->getProxyPort() << "</proxy-port>\n";
	t << "<additional-namespaces>" << encodeSGMLEntities( soapwidget->getAdditionalNamespaces() ) << "</additional-namespaces>\n";
	t << "<send-raw>" << soapwidget->getSendRaw() << "</send-raw>\n";
	t << "<show-headers>" << soapwidget->getShowHttpHeaders() << "</show-headers>\n";
	t << "<http-content-type>" << encodeSGMLEntities( soapwidget->getHttpContentType() ) << "</http-content-type>\n";
	t << "</soapsession>\n";
	t << "</qsoapman-config>\n";
	f.close();

	//e->setModified( FALSE );

	setCaption( filename );

	statusBar()->message( tr( "Session %1 saved" ).arg( filename ), 2000 );
}


void qsoapman::saveAs()
{
    QString fn = QFileDialog::getSaveFileName( QString::null, QString::null,
					       this );
    if ( !fn.isEmpty() ) {
	filename = fn;
	save();
    } else {
	statusBar()->message( tr("Saving aborted"), 2000 );
    }
}


// This was added by the KDevelop template.
//I did not delete it, because it might be useful later
/*
void qsoapman::print()
{
    // ###### Rewrite to use QSimpleRichText to print here as well
    const int Margin = 10;
    int pageNo = 1;

    if ( printer->setup(this) ) {		// printer dialog
	statusBar()->message( tr("Printing...") );
	QPainter p;
	if( !p.begin( printer ) )               // paint on printer
	    return;

	p.setFont( e->font() );
	int yPos	= 0;			// y-position for each line
	QFontMetrics fm = p.fontMetrics();
	QPaintDeviceMetrics metrics( printer ); // need width/height
						// of printer surface
	for( int i = 0 ; i < e->lines() ; i++ ) {
	    if ( Margin + yPos > metrics.height() - Margin ) {
		QString msg( "Printing (page " );
		msg += QString::number( ++pageNo );
		msg += ")...";
		statusBar()->message( msg );
		printer->newPage();		// no more room on this page
		yPos = 0;			// back to top of page
	    }
	    p.drawText( Margin, Margin + yPos,
			metrics.width(), fm.lineSpacing(),
			ExpandTabs | DontClip,
			e->text( i ) );
	    yPos = yPos + fm.lineSpacing();
	}
	p.end();				// send job to printer
	statusBar()->message( tr("Printing completed"), 2000 );
    } else {
	statusBar()->message( tr("Printing aborted"), 2000 );
    }
}
*/


void qsoapman::closeEvent( QCloseEvent* ce )
{
    //if ( !e->isModified() ) {
	ce->accept();
	return;
    //}

    switch( QMessageBox::information( this, tr( LONG_APP_NAME ),
				      tr("Do you want to save the changes"
				      " to the session?"),
				      tr("Yes"), tr("No"), tr("Cancel"),
				      0, 1 ) ) {
    case 0:
	save();
	ce->accept();
	break;
    case 1:
	ce->accept();
	break;
    case 2:
    default: // just for sanity
	ce->ignore();
	break;
    }
}


void qsoapman::about()
{
    QMessageBox::about( this, tr( LONG_APP_NAME ),
			tr( LONG_APP_NAME
			"\nVersion "
			VERSION
			"\n\n"
			"A GUI tool to send messages with the Simple Object Access Protocol (SOAP).\n\n"
			  "(c) 2004 by Jan Schreiber <mail@jschreiber.com> All rights reserved.\n"
			  "http://qsoapman.sourceforge.net\n\n"
			  "This program is licensed to you under the terms of the GNU General\n"
			  "Public License Version 2. For details, see the file LICENSE that came\n"
			  "with this software distribution.\n\n"
			  "The program is provided AS IS with NO WARRANTY OF ANY KIND,\n"
			  "INCLUDING THE WARRANTY OF DESIGN, MERCHANTABILITY AND\n"
			  "FITNESS FOR A PARTICULAR PURPOSE."  ) );
}


void qsoapman::aboutQt()
{
    QMessageBox::aboutQt( this, tr("qSOAPman - The Qt SOAP Manager") );
}
