// -------------------------------------------------------------------------
//   qSOAPman
//   Copyright (C) 2004 by Jan Schreiber
//   mail@jschreiber.com
//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program; if not, write to the
//   Free Software Foundation, Inc.,
//   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
// -------------------------------------------------------------------------

#include "soapsyntaxhighlighter.h"

#include <qstring.h>

#define INDENT "&nbsp;&nbsp;&nbsp;&nbsp;"

QString SOAPSyntaxHighlighter::encodeXMLEntities( const QString &s ) const
{
	QString data = s;
	data.replace( "&", "&amp;" ); // This _must_ come first
	data.replace( "<", "&lt;" );
	data.replace( ">", "&gt;" );
	data.replace( "\"", "&quot;" );
	data.replace( "'", "&apos;" );
	return data;
}

SOAPSyntaxHighlighter::SOAPSyntaxHighlighter()
{
	result = "";
	XMLProcessingMarkupColor = "darkblue";
	XMLElementColor = "darkcyan";
	XMLDataColor = "black";
	XMLDTDColor = "darkgreen";
	XMLCommentColor = "orange";
	XMLAttributeNameColor = "darkgreen";
	XMLAttributeValueColor = "darkred";
	XMLErrorColor = "red";
	XMLCDATATagColor = "darkgreen";
	XMLCDATAColor = "magenta";
}

bool SOAPSyntaxHighlighter::startDocument()
{
	indent = "";
	startedElement = false;
	inCDATA = false;
	return TRUE;
}

bool SOAPSyntaxHighlighter::startCDATA()
{
	if ( startedElement )
		result += "&gt;";
	result += "<br>" + indent + "<font color=\"" + XMLCDATATagColor + 
	          "\">&lt;![CDATA[</font>";
	indent += INDENT;
	startedElement = false;
	inCDATA = true;
	return TRUE;
}

bool SOAPSyntaxHighlighter::endCDATA()
{
	indent.remove( 0, QString( INDENT ).length() );
	result += "<br>" + indent + "<font color=\"" + XMLCDATATagColor + 
		"\">]]&gt;</font>";
	startedElement = false;
	inCDATA = false;
	return TRUE;
}

bool SOAPSyntaxHighlighter::comment( const QString & ch )
{
	if ( startedElement )
	{
		result += "&gt;";
	}
	// copy because ch ist const
	QString comment = ch;
	comment.replace( "\n", "<br>" + indent );
	result += "<br>" + indent + "<font color=\"" + XMLCommentColor +
		"\">&lt;!--" + comment + "--&gt;</font>";
	startedElement = false;
	return TRUE;
}

bool SOAPSyntaxHighlighter::startElement( const QString&, const QString&, const QString& qName,
                                    const QXmlAttributes& attributes )
{
	//indent += INDENT;
	if ( startedElement )
	{
		result += "&gt;</font>";
	}
	result +="<br>";
	result += indent;
	
	result += "<font color=\"" + XMLElementColor + "\">&lt;";
	result += qName;
	if ( attributes.length() > 0 )
	{
		for ( int i = 0 ; i < attributes.length(); i++ )
		{
			result += " <font color=\"" + XMLAttributeNameColor + "\">"
				+ attributes.qName( i ) + "</font>";
			result += "<font color=\"" + XMLProcessingMarkupColor + "\">=</font>";
			result += "<font color=\"" + XMLAttributeValueColor + "\">\""
				+ attributes.value( i ) + "\"</font>";
		}
	}
	indent += INDENT;
	startedElement = true;
	return TRUE;
}

bool SOAPSyntaxHighlighter::endElement( const QString&, const QString&, const QString& qName )
{
	indent.remove( 0, QString( INDENT ).length() );
	if ( startedElement )
	{
		startedElement = false;
		result += " /&gt;</font>";
		return TRUE;
	}
	result += "<br>" + indent + "<font color=\"" + XMLElementColor + "\">&lt;/";
	result += qName + "&gt;</font>";
	
	startedElement = false;
	return TRUE;
}

bool SOAPSyntaxHighlighter::processingInstruction ( const QString & target, const QString & data )
{
	result += "<font color=\"" + XMLProcessingMarkupColor + "\">&lt;?</font><font color=\""
		 + XMLDTDColor + "\">";
	result += target + " ";
	result += data;
	result += "</font><font color=\"" + XMLProcessingMarkupColor + "\">?&gt;</font>";
	startedElement = false;

	return TRUE;
}

bool SOAPSyntaxHighlighter::characters ( const QString & ch )
{
	QString str = ch.stripWhiteSpace();
	if ( str.isEmpty() ) return TRUE;

	if ( startedElement )
	{
		result += "&gt;</font>";
	}
	// note that we have to call encodeSGMLEntities twice:
	// once to encode the character string and once because the edit field interpretes
	// this as html ( which would undo the first encoding )
	result += "<br>" + indent + "<font color=\""
		+ (inCDATA ? XMLCDATAColor : XMLDataColor) + " \">" 
		+ encodeXMLEntities( encodeXMLEntities(str) ).replace( "\n", "<br>" + indent ) + "</font>";
	startedElement = false;

	return TRUE;
}

QString SOAPSyntaxHighlighter::getResult()
{
	return result;
}


